 PROGRAM C.F
*******************************************************************************
* Bp.Es C.F - A program to create a new file.
*
*  Copyright 2008 Rush Flat Software
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*      http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
* Questions or comments to Rush Flat Software
* Email: brian at rushflat dot co dot nz
*
* --------------------------------------------------------------------------- *
*
* Author : BSS
* Created: 30 Dec 2008
* Updated: 28 Mar 2009
* Version: 1.0.1
*
* Syntax
* ======
* C.F filename [filetype][[s:]itemsize][[d:]dictitems][n:dataitems]
*
* filename - filename to create both dict and data portions
*          - DICT,filename to create only the dictionary portion
*          - dictname,dataname to create only the data portion
* filetype - 'directory' or 'hashed'
* itemsize - the average item size of DATA items
* dictitems - the number of dictionary items
* dataitems - the number of data items
*
* Notes
* =====
* The actual syntax used is flavour dependent:
* OpenQM/Universe:
*   C.F filename [filetype][[s:]itemsize]
* mvBASE
*   C.F filename [filetype][[s:]itemsize][[d:]dictitems][n:dataitems]
*
* The difference in syntax is because C.F creates dynamic files for OpenQM and
* UniVerse, but static hashed files for mvBASE. Static hashed files require more
* information for creation.
*
* If filetype is not specified, a hashed file is assumed.
* mvBASE does not support directory files. The file creation subroutine will
* return an error if a directory file is specified using C.F.
*
* Dictionary items are assumed to have an average size of 50 bytes.
*
* If no data item size is specified, a default value of 200 bytes is assumed.
*
* For mvBASE, if no item counts are specified, 10 items per file are assumed.
* NOTE this will result in default file sizing of 1,1 which will be dramatically
* undersized for most data files.
*
* Version history
* ---------------
* v.1.0.0 - Dec 2008 - Original release
* v.1.0.1 - Mar 2009 - mvBASE functionality added
*
*
* --------------------------------------------------------------------------- *
*
$CATALOGUE GLOBAL;                               *  %%QM%%
$MODE UV.LOCATE;                                 *  %%QM%%
$MODE PICK.ERRMSG;                               *  %%QM%%

 s= @SENTENCE;                                   *  %%QM%%%%UV%%
 s = SENTENCE();                                 *  %%MV%%
 s = TRIM(OCONV(s, 'MCU'))
 CONVERT ' ' TO @AM IN s
 filename = s<2>
 IF filename = '' THEN GOTO usage

 hasparts = INDEX(filename, ',', 1)
 IF hasparts THEN
   dictpart = FIELD(filename, ',', 1)
   datapart = FIELD(filename, ',', 2)
   IF dictpart = 'DICT' THEN
     filetype = 'DICT'
   END ELSE
     filetype = 'DATA'
   END
 END ELSE
   filetype = 'DICTDATA'
 END

 dc = DCOUNT(s, @AM)
 IF dc GT 5 THEN GOTO usage

 filetype = 'HASHED'
 itemsize = ''
 numitems = ''

 IF dc > 2 THEN
   FOR ii = 3 TO dc
     thisword = s<ii>
     BEGIN CASE
       CASE thisword = 'DIRECTORY' OR thisword = 'HASHED'
         filetype = thisword
       CASE thisword[1,2] = 'S:'
         temp = thisword[3,99]
         IF NUM(temp) THEN itemsize = temp
       CASE thisword[1,2] = 'D:'
         temp = thisword[3,99]
         IF NUM(temp) THEN numitems<1> = temp
       CASE thisword[1,2] = 'N:'
         temp = thisword[3,99]
         IF NUM(temp) THEN numitems<2> = temp
       CASE NUM(thisword) AND thisword > 0
         BEGIN CASE
           CASE itemsize = ''
             itemsize = thisword
           CASE numitems<1> = '' AND filetype NE 'DATA'
             numitems<1> = thisword
           CASE numitems<2> = '' AND filetype NE 'DICT'
             numitems<2> = thisword
           CASE 1
             CRT 'Unknown parameter: ':thisword
             GOTO usage
         END CASE
       CASE 1
         CRT 'Unknown word: ':thisword
         GOTO usage
     END CASE
   NEXT ii
 END

 location = ''
 numitems = ''
 err = ''
 CALL SY.CREATE.FILE(filename, filetype, location, itemsize, numitems, err)

 IF err > '' THEN
   CRT 'Error: ':err
 END

 STOP
*
* --------------------------------------------------------------------------- *
*
usage:

 CRT 'Usage: C.F filename [filetype] [[s:]itemsize]'  ;* %%QM%%%%UV%%
 CRT 'Usage: C.F filename [filetype] [[s:]itemsize] [[d:]dictitems] [[n:]dataitems]' ;* %%MV%%
 CRT '       where: filetype = DIRECTORY or HASHED'
 CRT '              itemsize = average data item size'
 CRT '              dictitems = expected number of items in dictionary'   ;* %%MV%%
 CRT '              dataitems = expected number of items in data file'    ;* %%MV%%
 STOP
*
* --------------------------------------------------------------------------- *
*
 END
